<?php
/*
Plugin Name: Yahoo Boss
Plugin URI: http://www.picklewagon.com/wordpress/yahoo-boss-plugin
Description: Adds a Yahoo Boss Search Engine to your WordPress install.
Author: Josh Harrison
Version: 0.5
Author URI: http://www.picklewagon.com
*/

/*  Copyright 2009
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
* Guess the wp-content and plugin urls/paths
*/
if ( !defined('WP_CONTENT_URL') )
	define( 'WP_CONTENT_URL', get_option('siteurl') . '/wp-content');
if ( !defined('WP_CONTENT_DIR') )
	define( 'WP_CONTENT_DIR', ABSPATH . 'wp-content' );

if (!defined('WP_PLUGIN_URL'))
	define('WP_PLUGIN_URL', WP_CONTENT_URL . '/plugins/');
if (!defined('PLUGIN_PATH'))
	define('PLUGIN_PATH', WP_CONTENT_DIR . '/plugins/');

if (!class_exists('pw_yahoo_boss')) {
	class pw_yahoo_boss {
		/**
		* @var string The options string name for this plugin
		*/
		var $fullPluginName = 'Yahoo Boss';
		
		/**
		* @var string The options string name for this plugin
		*/
		var $optionsName = 'pw_yahoo_boss_options';
		
		/**
		* @var string $localizationDomain Domain used for localization
		*/
		var $localizationDomain = "pw_yahoo_boss";
		
		/**
		* @var string $pluginurl The path to this plugin
		*/
		var $pluginurl = '';
		/**
		* @var string $pluginurlpath The path to this plugin
		*/
		var $pluginpath = '';
		
		/**
		* @var array $options Stores the options for this plugin
		*/
		public $options = array();
		
		//Class Functions
		/**
		* PHP 4 Compatible Constructor
		*/
		function pw_yahoo_boss() {
			$this->__construct();
		}
		
		/**
		* PHP 5 Constructor
		*/
		function __construct() {
			// Language Setup
			$locale = get_locale();
			$mo = dirname(__FILE__) . "/languages/" . $this->localizationDomain . "-".$locale.".mo";
			load_textdomain($this->localizationDomain, $mo);
			
			// "Constants" setup
			$this->pluginurl = WP_PLUGIN_URL . '/' . dirname(plugin_basename(__FILE__)).'/';
			$this->pluginpath = PLUGIN_PATH . dirname(plugin_basename(__FILE__)).'/';
			
			// Initialize the options
			$this->getOptions();
			
			// Actions
			add_action("admin_menu", array(&$this, 'admin_menu_link'));
			add_action("wp_head", array(&$this, 'add_css'));
			
			// Shortcodes
			add_shortcode('boss_search_form', array(&$this, 'show_search_form'));
			add_shortcode('boss_search_results', array(&$this, 'show_boss_search_results'));
		}
		
		/**
		* Retrieves the plugin options from the database.
		* @return array
		*/
		function getOptions() {
			if (!$theOptions = get_option($this->optionsName)) {
				$theOptions = array(
					'pw_yahoo_boss_results_per_page' => 10,
					'pw_yahoo_boss_format'           => 'xml',
					'pw_yahoo_boss_domains'          => null,
				);
				update_option($this->optionsName, $theOptions);
			}
			$this->options = $theOptions;
		}
		
		/**
		* Saves the admin options to the database.
		*/
		function saveAdminOptions(){
			update_option($this->optionsName, $this->options);
		}
		
		/**
		* @desc Adds the options subpanel
		*/
		function admin_menu_link() {
			add_options_page($this->fullPluginName, $this->fullPluginName, 10, 'yahoo-boss-config', array(&$this, 'admin_options_page'));
			add_filter( 'plugin_action_links', array(&$this, 'filter_plugin_actions'), 10, 2 );
		}
		
		/**
		* @desc Adds the Settings link to the plugin activate/deactivate page
		*/
		function filter_plugin_actions($links, $file) {
			static $this_plugin;
			
			if (!$this_plugin) {
				$this_plugin = plugin_basename(__FILE__);
			}
		
			if( $file == $this_plugin ){
				$settings_link = '<a href="options-general.php?page=' . 'yahoo-boss-config' . '">' . __('Settings') . '</a>';
				array_unshift( $links, $settings_link ); // before other links
			}
			
			return $links;
		}

		function saveDomains($domains) {
			$d = explode("\n", $domains);
			for ($i = 0; $i < count($d); $i++) {
				$d[$i] = trim($d[$i]);
				$d[$i] = str_replace("http://", "", $d[$i]);
			}
			return $d;
		}
		
		/**
		* Adds settings/options page
		*/
		function admin_options_page() { 
			if($_POST['pw_yahoo_boss_save']){
				$this->options['pw_yahoo_app_id'] = $_POST['pw_yahoo_app_id'];                   
				$this->options['pw_yahoo_boss_results_per_page'] = $_POST['pw_yahoo_boss_results_per_page'];
				$this->options['pw_yahoo_boss_format'] = $_POST['pw_yahoo_boss_format'];
				$this->options['pw_yahoo_boss_domains'] = $this->saveDomains($_POST['pw_yahoo_boss_domains']);
				
				$this->saveAdminOptions();
				
				echo '<div class="updated"><p>Success! Your changes were sucessfully saved!</p></div>';
			}
			
			if ($this->options['pw_yahoo_boss_domains']) {
				$domains = implode("\n", $this->options['pw_yahoo_boss_domains']);
			} else {
				$domains = null;
			}
?>
			<div class="wrap">
				<h2><?php echo $this->fullPluginName; ?></h2>
				<form method="post" id="pw_yahoo_boss_options">
					<table width="100%" cellspacing="2" cellpadding="5" class="form-table">
						<tr valign="top">
							<th scope="row"><label for="pw_yahoo_app_id"><?php _e('Yahoo App ID:', $this->localizationDomain); ?></label></th>
							<td><input name="pw_yahoo_app_id" type="text" id="pw_yahoo_app_id" value="<?php echo $this->options['pw_yahoo_app_id'] ;?>" class="regular-text code<?php if (defined('YAHOO_APP_ID')) print ' disabled'; ?>" <?php if (defined('YAHOO_APP_ID')) print 'disabled="disabled" '; ?>/>
							<span class="setting-description"><a href="http://developer.yahoo.com/search/boss">Get your Yahoo! App ID</a></span></td>
						</tr>
						<tr valign="top">
							<th scope="row"><label for="pw_yahoo_boss_results_per_page"><?php _e('Results per Page:', $this->localizationDomain); ?></label></th>
							<td><input name="pw_yahoo_boss_results_per_page" type="text" id="pw_yahoo_boss_results_per_page" value="<?php echo $this->options['pw_yahoo_boss_results_per_page'] ;?>"/></td>
						</tr>
						<tr valign="top">
							<th scope="row"><?php _e('Format:', $this->localizationDomain); ?></th>
							<td>
								<input name="pw_yahoo_boss_format" type="radio" id="pw_yahoo_boss_format_xml" value="xml" <?php if ($this->options['pw_yahoo_boss_format'] == 'xml') print 'checked="checked" '?>/> <label for="pw_yahoo_boss_format_xml">XML</label><br />
								<input name="pw_yahoo_boss_format" type="radio" id="pw_yahoo_boss_format_json" value="json" <?php if ($this->options['pw_yahoo_boss_format'] == 'json') print 'checked="checked" '?>/> <label for="pw_yahoo_boss_format_json">JSON (PHP 5 required)</label>
							</td>
						</tr>
						<tr valign="top">
							<th scope="row"><label for="pw_yahoo_boss_domains"><?php _e('Domains to Search:', $this->localizationDomain); ?></label></th>
							<td><textarea name="pw_yahoo_boss_domains" id="pw_yahoo_boss_domains" cols="50" rows="8"><?php echo $domains;?></textarea></td>
						</tr>
					</table>
					<p class="submit"><input type="submit" class="button-primary" name="pw_yahoo_boss_save" value="Save Changes" /></p>
				</form>
			</div>
<?php
		}
		
		function add_css() {
			$default_stylesheet = $this->pluginurl . 'search-form.css';
			
			$stylesheet = apply_filters('pw_boss_stylesheet', $default_stylesheet);
			
			print '<link rel="stylesheet" type="text/css" href="' . $stylesheet . '" />';
		}

		function getDomains() {
			// Comma-separated list of domains to search in
			$domains = implode(",", $this->options['pw_yahoo_boss_domains']);
			return $domains;
		}
		
		function getFormat() {
			return $this->options['pw_yahoo_boss_format'];
		}
		
		// Finds [boss_search_results] shortcode and produces the search results in the content
		function show_boss_search_results() {
			pw_boss_search_results();
		}
		
		// Finds [boss_search_form] shortcode and produces the search results in the content
		function show_search_form() {
			pw_boss_search_form();
		}
		
		function get_yahoo_app_id() {
			$app_id = false;
			if (defined('YAHOO_APP_ID')) {
				$app_id = YAHOO_APP_ID;
			} else {
				$app_id = $this->options['pw_yahoo_app_id'];
			}
			
			return $app_id;
		}
	} //End Class
} //End if class exists statement

//instantiate the class
if (class_exists('pw_yahoo_boss')) {
    $pw_yahoo_boss = new pw_yahoo_boss();
}

include_once('yahoo-boss-results.php');
include_once('template-functions.php');

// give the admin a warning if the yahoo app id has not been set
if ( !$pw_yahoo_boss->get_yahoo_app_id() && !isset($_POST['pw_yahoo_boss_save']) ) {
	function pw_yahoo_boss_warning() {
		echo "<div id='yahoo-boss-warning' class='updated fade'><p><strong>".__('Yahoo! BOSS is almost ready.')."</strong> ".sprintf(__('You must <a href="%1$s">add your Yahoo! app ID</a> for it to work.'), "options-general.php?page=yahoo-boss-config")."</p></div>";
	}
	add_action('admin_notices', 'pw_yahoo_boss_warning');
	return;
}
?>